/** \file Doc-Simu.h
    \brief Documentation detaillee de la partie simulation de l'application : page associee au code. 
*/

/*! \page Doc-Simu  Documentation dtaille de la partie simulation de l'application
 Cette page explique :
 - \ref sec1 (point de vue gnral)
 	- \ref subsection11 : objet essentiel permettant de lier tous les objets de la scne
 	- \ref subsection12 
 	
 - \ref sec2
 	- \ref subsection21	
 	- \ref subsection22
 	
 - \ref sec3
 
 - \ref sec4
 	- \ref subsec41
 	- \ref subsec42
 	- \ref subsec43
 	- \ref subsec44
  
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

 \section sec1 Fonctionnement de la partie simulation de l'application 

\subsection subsection11 Le graphe de scne

 Le graphe de scne dfini par la classe \ref Scene permet de grer les diffrents objets de la scne :
 - Chaque objet que nous souhaitons simuler doit hrit d'une classe nomme \ref Noeud 
 - La classe \ref Scene contient la liste des \ref Noeud de la scne
 - Les diffrentes mthodes de \ref Scene vont permettre d'appeler les mthodes correspondantes pour chacun des objets \ref Noeud attach  la scne.

 Les mthodes de la classe \ref Noeud doivent ainsi obligatoirement tre implmentes dans les classes relatives aux diffrents objets de la scne (mme si elles ne font rien). 


\subsection subsection12 Programme principal

Le programme principal \ref main contient un objet \ref Scene correspondant au graphe de la scne. Cet objet contient une liste de tous les objets \ref ObjetSimule qui vont tre simuls, attachs  la scne au sein du programme principal (voir \ref subsection11).

Une fois tous les objets attachs au graphe de scne, les mthodes suivantes de l'objet \ref Scene sont invoques (rpercutant leur appel sur tous les objets attachs  la scne) :
	- init() : initialisation des objets simuls.
	- Simulation() : simulation des objets de la scne.

A noter que, la gravit et la viscosit du milieu sont dfinies dans la \ref Scene, c'est--dire que leurs proprits sont identiques pour chacun des objets simuls. Leurs caractristiques sont fournies au lancement de l'application via le fichier de donnes.

\section sec2  Structures de donnes pour la partie simulation
 
\subsection subsection21 Les types 

 Les principaux types de structures de donnes utiliss dans le module de simulation sont les suivants :
	- std::vector<float> : 
		- Essentiellement utilis pour stocker les masses des particules
		
	- std::vector<Coord> :
		- avec \ref Coord la structure dfinissant des coordonnes 3D
		- Essentiellement utilis pour stocker les positions, vitesses, acclrations et forces des particules
			
 
 \subsection subsection22 Les classes 

L'application permet l'animation d'objets 3D dformables. Ces objets sont ainsi reprsents par un systme masses-ressorts c'est--dire par un ensemble de particules connectes entre elles par des ressorts.	

Les principales classes dfinies pour modliser un objet sont ainsi les suivantes :

- \ref Spring : 
	- permet de stocker les caractristiques physiques des ressorts (raideur, amortissement, longueur au repos et viscosit)

- \ref Particule :
	- dfinition physique d'une particule
	- est constitue d'un identifiant, d'une position, et de la liste des ressorts auxquels la particule est relie

- \ref Ressort :
	- dfinition gomtrique d'un ressort
	- est constitu d'un identifiant, d'un \ref Spring contenant ces proprits, et des deux indices des particules auxquelles le ressort est reli  

 - \ref Mesh :
 	- dfinition d'un maillage
 	- contient une liste de \ref Ressort, une liste de \ref Particule 	

\section sec3 Reprsentation d'un objet de la scne
	
Les principales structures de donnes des objets simuls sont les suivantes :
	- Maillage
	- P
	- V
	- A
	- Force
	- M
	
La structure Maillage dfinit le \ref Mesh relatif  l'objet simul. Ainsi, tout objet de la scne est discrtis en un ensemble de \ref Particule qui sont relies entre elles par des \ref Ressort. A noter que pour le moment, tous les ressorts d'un mme objet ont les mmes caractristiques physiques dfinies par l'utilisateur. Il sera possible d'ajouter ultrieurement ces caractristiques pour chacun des ressorts constituant le maillage en les mettant par exemple dans un fichier de donnes. Ce Maillage est cr lors de l'initialisation de l'objet simul, notamment  partir du fichier des facettes fourni en paramtre de l'objet  simuler. Ce maillage est utilis pour connatre les interactions qui existent entre les particules d'un objet donn.	
		
La structure P contient les coordonnes de positions des particules constituant l'objet  simuler. Cette structure est initialise  partir du fichier de donnes correspondant rattach  l'objet. 

La structure V contient les coordonnes de vitesses des particules constituant l'objet  simuler. Les lments de cette structure sont initialiss  zro au dpart. 

La structure A contient les coordonnes des acclrations des particules constituant l'objet  simuler. Les lments de cette structure sont initialiss  zro au dpart. 

La structure Force contient les coordonnes des forces appliques aux particules constituant l'objet  simuler. Les lments de cette structure sont initialiss  zro au dpart.

La structure M contient les coordonnes de positions des particules constituant l'objet  simuler. Cette structure est initialise  partir du fichier de donnes correspondant rattach  l'objet.

 \section sec4 Algorithmes de la simulation
 
 \subsection subsec41 Boucle de la simulation
 
 A chaque pas de temps, les calculs suivants sont effectus pour un objet donn : 
 - \ref subsec42 appliques aux particules constituant l'objet
 - \ref subsec43 des particules
 - \ref subsec44 des particules
 
  \subsection subsec42 Calcul des forces

 Les forces f appliques aux particules constituant un objet sont dues  la gravit du milieu ainsi qu'aux interactions des autres particules constituant l'objet. La force applique   la ime particule du systme peut ainsi tre formule par :
 
\f$ \vec{f}_i(t) = \sum_{j\mid(i,j)\in   E}  \left[\vec{f}^{e}_{i,j}(t) + \vec{f}^{v}_{i,j}(t) \right] \: +  m_i \vec{g} +\:  \vec{f}_{externe}(t),\f$

avec :

\f$
\left\{
\begin{array}{lcl}
\vec{f}^{e}_{i,j}(t) &=& k_{ij} \left(\|x_i(t) - x_j(t)\| - l_{ij}\right)\;  \vec{u}_{ij}(t),\\
\vec{f}^{v}_{i,j}(t) &=& {\nu}_{ij}  \left(v_i(t) -  v_j(t)\right) \; \vec{u}_{ij}(t) \; \vec{u}_{ij}(t),
\end{array} 
\right.
\f$

o :
- E  reprsente l'ensemble  des  artes  du  systme   particules,
- \f$k_{ij}\f$ est la constante de raideur des   ressorts,
- \f${\nu}_{ij}\f$ est la constante d'amortissement des   ressorts,
- \f$l_{ij}\f$ est la longueur  du  ressort  au repos reliant la  particule  i et  la particule  j,  
- \f$\vec{g}\f$  reprsente le vecteur de la   gravit,   
- et \f$\vec{u}_{ij}(t)\f$ est un vecteur unitaire dfini par 
		\f$ \vec{u}_{ij}(t) = \frac{x_i(t) - x_j(t)}{\|x_i(t) - x_j(t)\|} \f$. 
  
 Pour des raisons d'implmentation, les forces dues  la gravit sont en fait ajoutes dans le calcul des acclrations.

 
  \subsection subsec43 Calcul des acclrations
Les acclrations des particules sont obtenues  partir de leurs forces en utilisant la loi fondamentale de la dynamique, avec \f$ a_i = \frac{f_i}{m_i} \f$ o \f$a_i\f$ est l'acclration de la particule i, \f$f_i \f$ la force applique sur cette particule, et \f$m_i \f$ sa masse.

C'est au sein de cette mthode que le vecteur des forces est rinitialis. 
   
   
   \subsection subsec44 Calcul des vitesses et des positions

 Les vitesses et les positions des particules sont obtenues en utilisant la mthode explicite de Leap-Frog Verlet :   
 
 \f$
\begin{array}{lr}
\left\{
\begin{array}{ccl}
v(t_0)   &=&   v_0\\
v(\frac{h}{2}) &=& v_0 + \frac{h}{2}v'_0\\ 
x(t_0) &=& x_0
\end{array}
\right., 
& 
\left\{
\begin{array}{ccl}
v(t+\frac{h}{2}) &=& v(t-\frac{h}{2}) + hv'(t) \\ 
x(t+h) &=& x(t) + hv(t+\frac{h}{2}) 
\end{array}
\right..
\end{array}
\f$


 
 */