/** \file Text2Bin.cpp
  Conversion Text vers Binaire des fichiers de donnees (points, masses, facettes et texture).
*/

#include <stdio.h>
#include <math.h>
#include <vector>
#include <iostream>
#include <fstream>

#include "Donnees.h"


/**
 * Initialisation des tableaux et construction du maillage a partir des donnees lues dans le fichier.
 */
int main(int argc, char** argv)
{  
  // Verification des arguments
  if (argc <3) 
    {
      std::cout << "Usage " << argv[0] << " fich_coord fich_facet (fich_mass) (fich_text)" << std::endl;	
      exit(1);
    }

  // Noms des fichiers de donnees textes : fich_coord fich_facet
  std::string n_fich_coord = argv[1];
  std::string n_fich_facet = argv[2];
  
  // Noms des fichiers de donnees binaires : fich_coord.bin fich_facet.bin 
  std::string n_fich_coord_bin = n_fich_coord + ".bin";
  std::string n_fich_facet_bin = n_fich_facet + ".bin";
  
  // Lecture des fichiers textes
  std::ifstream fich_coord(n_fich_coord.c_str()) ;
  std::ifstream fich_facet(n_fich_facet.c_str()) ;
  
  // Ecriture des fichiers binaires
  std::ofstream fich_coord_bin(n_fich_coord_bin.c_str()) ;
  std::ofstream fich_facet_bin(n_fich_facet_bin.c_str()) ;


  /** Ouverture des fichiers de donnees **/  
  /* Fichier des points */
  // Fichier texte
  if ((fich_coord.eof()) || (! fich_coord)) 
    {
      std::cout << "Erreur d ouverture du fichier de donnees des points " << n_fich_coord << std::endl;
      exit(1);
    }

  // Fichier binaire
  if ((fich_coord_bin.eof()) || (! fich_coord_bin)) 
    {
      std::cout << "Erreur d ouverture du fichier binaire de donnees des points " 
		<< n_fich_coord_bin << std::endl;
      exit(1);
    }

  /* Fichier des faceSet */
  // Fichier texte
  if ((fich_facet.eof()) || (! fich_facet)) 
    {
      std::cout << "Erreur d ouverture du fichier de donnees des faceSet " 
		<< n_fich_facet << std::endl;
      exit(1);
    }

  // Fichier binaire
  if ((fich_facet_bin.eof()) || (! fich_facet_bin)) 
    {
      std::cout << "Erreur d ouverture du fichier binaire de donnees des faceSet " 
		<< n_fich_facet_bin << std::endl;
      exit(1);
  }

   
  /** Positions des particules **/
  /* Nombre de particules */
  int nbparticules;
  Coord coord;
  fich_coord >> nbparticules;
  Coord* allparticules = new Coord[nbparticules];

  /* Positions des particules */
  for(int i=0; i<nbparticules; ++i)
    {
      // Lecture des valeurs 
      fich_coord >> coord.x;
      fich_coord >> coord.y;
      fich_coord >> coord.z;
      allparticules[i] = coord;
      
    }//for_i
  
  /* Ecriture en binaire */
  std::cout << "#particules=" << nbparticules << std::endl;
  fich_coord_bin.write( (const char*)&nbparticules, sizeof(int) );
  fich_coord_bin.write( (const char*)allparticules, sizeof(Coord)*nbparticules );

 
  /** Lecture des facettes du maillage **/
  std::vector<Facet> allfacets;
  while (!fich_facet.eof())
    {
      // Lecture de la facette fi, fj et fk 
      Facet facet;
      fich_facet >> facet.fi;
      fich_facet >> facet.fj;
      fich_facet >> facet.fk;  
      allfacets.push_back(facet);
    }

  /* Ecriture en binaire */
  int nbfacets = allfacets.size()-1;
  std::cout << "#facets=" << nbfacets << std::endl;
  fich_facet_bin.write( (const char*)&nbfacets, sizeof(int) );
  fich_facet_bin.write( (const char*)&allfacets[0], sizeof(Facet)*nbfacets );


	/** Masses **/
	if (argc >= 4)
	{
		// Noms du fichier de donnees texte :  fich_mass
		std::string n_fich_mass = argv[3];

		// Noms du fichier de donnees binaire : fich_mass.bin
 		std::string n_fich_mass_bin = n_fich_mass + ".bin";

 		// Lecture des fichiers textes
		std::ifstream fich_mass(n_fich_mass.c_str()) ;

		// Ecriture des fichiers binaires
 		std::ofstream fich_mass_bin(n_fich_mass_bin.c_str()) ;


 		/* Fichier des masses */
  		// Fichier texte
  		if ((fich_mass.eof()) || (! fich_mass)) 
    	{
      		std::cout << "Erreur d ouverture du fichier de donnees des masses " 
			<< n_fich_mass << std::endl;
      		exit(1);
    	}

 		// Fichier binaire
		if ((fich_mass_bin.eof()) || (! fich_mass_bin)) 
   		 {
   			 std::cout << "Erreur d ouverture du fichier binaire de donnees des masses " 
			  << n_fich_mass_bin << std::endl;
     		 exit(1);
 		 }

		 /** Masses des particules **/
 		 std::vector<Precision> allmass;
  		while (!fich_mass.eof())
   		 {
    		  // Lecture de la masse
    		  Precision mass;
     		 fich_mass >> mass;
     		 allmass.push_back(mass);
    	}

  		/* Ecriture en binaire */
 		int nbmass = allmass.size()-1;
  		std::cout << "#mass=" << nbmass << std::endl;
  		fich_mass_bin.write( (const char*)&nbmass, sizeof(int) );
  		fich_mass_bin.write( (const char*)&allmass[0], sizeof(Precision)*nbmass );
}


  /** Textures **/
  if (argc >= 5)
    {
      // Noms des fichiers de donnees textes : fich_coord fich_facet fich_mass fich_text
      std::string n_fich_text = argv[4];

      // Noms des fichiers de donnees binaires : fich_coord.bin fich_facet.bin fich_mass.bin fich_text.bin
      std::string n_fich_text_bin =  n_fich_text + ".bin";

      // Lecture des fichiers textes
      std::ifstream fich_text(n_fich_text.c_str()) ;

      // Ecriture des fichiers binaires
      std::ofstream fich_text_bin(n_fich_text_bin.c_str()) ;

      /** Ouverture des fichiers de donnees **/
      /* Fichier des textures */
      // Fichier binaire
      if ((fich_text.eof()) || (! fich_text)) 
	{
	  std::cout << "Erreur d ouverture du fichier de donnees des textures " 
		    << n_fich_text << std::endl;
	  exit(1);
	}
      
      // Fichier binaire
      if ((fich_text_bin.eof()) || (! fich_text_bin)) 
	{
	  std::cout << "Erreur d ouverture du fichier binaire de donnees des textures " 
		    << n_fich_text_bin << std::endl;
	  exit(1);
	}

      /** Textures **/
      std::vector<Texture> alltext;
      while (!fich_text.eof())
	{
	  // Lecture des textures 
	  Texture text;
	  fich_text >> text.a;
	  fich_text >> text.b;
	  alltext.push_back(text);
	}

      /* Ecriture en binaire */
      int nbtext = alltext.size()-1;
      std::cout << "#text=" << nbtext << std::endl;
      fich_text_bin.write( (const char*)&nbtext, sizeof(int) );
      fich_text_bin.write( (const char*)&alltext[0], sizeof(Texture)*nbtext );
    }

  return 0;
  
}//main
