/*
 * Scene.h : Scene 3D
 * Copyright (C) 2007 Florence Zara, LIRIS
 *               florence.zara@liris.univ-lyon1.fr
 *               http://liris.cnrs.fr/florence.zara/
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/** \file Scene.h
\brief Definition du graphe de scene 
*/


#ifndef SCENE_H
#define SCENE_H


/** Librairie de base **/
#include <list>
#include <string>
#include <iostream>

/** Pour l affichage **/
#ifndef WIN32
#include <GL/gl.h>
#include <GL/glu.h>
#endif

/** Fichiers de l application **/
#include "Noeuds.h"
#include "Donnees.h"

/**
* \brief Type liste de noeuds.
 */
typedef std::list<Noeud*> ListeNoeuds;


/**
* \brief Scene 3D (visualisation et animation).
 */
class Scene 
{
public:
	
	/*! Constructeur */
	Scene(std::string Fichier_Param);
	
	/*! Ajoute un enfant dans le graphe de scene */
	void attache(Noeud *n);
	
	/*! Renvoie le nom des enfants */
	void getName();
	
	/*! Lecture des parametres de l execution relatfs a la scene */
	void Param(std::string Fichier_Param);
	
	/*! Modification de la gravite de la scene */
	void setGravite(Coord g);
	
	/*! Modification de la viscosite du milieu */
	void setVisco(float visco);
	
	/*! Action d initialisation */
	void init();
		
	/*! Affichage des enfants */
	void affiche();
	
	/*! Action de simulation */
	void Simulation(int Tps);
		
	/*! Destructeur */
	virtual ~Scene(){};
	
	
public:
		
	/// Liste de pointeurs sur les enfants	
	ListeNoeuds _enfants;  
	
	/// Gravite de la scene 
	Coord _g;
	
	/// Viscosite du milieu
	float _visco;
	
	/// Nombre d iterations de la boucle de simulation 
	int _nb_iter;
	
};


#endif
